"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const classNames = require("classnames");
const PropTypes = require("prop-types");
const React = require("react");
const react_transition_group_1 = require("react-transition-group");
const screen_1 = require("../screen");
class Navigator extends React.Component {
    constructor(props, context) {
        super(props);
        this.renderScreen = (screen, index) => {
            const { active } = this.state;
            const comp = this.context.registry.getLocalComponent(screen.name);
            if (!comp) {
                // TODO(jian.guo): May be render NotFound screen in production
                throw new Error(`Screen not found: ${screen}`);
            }
            let props = {
                classNames: '',
                timeout: 250,
            };
            const { animationType, transitionProps } = comp;
            if (animationType && animationType !== screen_1.AnimationType.NONE) {
                props.classNames = `screen-animation-${animationType}`;
            }
            if (screen.transitionStyles) {
                return (React.createElement(react_transition_group_1.Transition, { key: `${screen}_${index}`, timeout: { enter: 0, exit: 300 }, enter: true, exit: true, mountOnEnter: true, unmountOnExit: true }, (state) => {
                    return (React.createElement(screen_1.Screen, { index: index, component: comp, status: active === index ? screen_1.ScreenStatus.ACTIVE : screen_1.ScreenStatus.IDLE, navigator: this, params: screen.params, style: screen.transitionStyles[state] }));
                }));
            }
            return (React.createElement(react_transition_group_1.CSSTransition, Object.assign({ appear: true, mountOnEnter: true, unmountOnExit: true, key: `${screen}_${index}` }, props, transitionProps),
                React.createElement(screen_1.Screen, { index: index, component: comp, status: active === index ? screen_1.ScreenStatus.ACTIVE : screen_1.ScreenStatus.IDLE, navigator: this, params: screen.params })));
        };
        this.finishLaunching = () => {
            const { homeScreen, initialScreens } = this.props;
            const { screens } = this.state;
            if (initialScreens.length) {
                this.setState({
                    screens: [homeScreen, ...initialScreens],
                    active: initialScreens.length + 1,
                }, () => {
                    this.replaceState(initialScreens[initialScreens.length - 1]);
                });
            }
            else {
                this.push(homeScreen.name, homeScreen.params);
            }
        };
        this.push = (name, params, transitionStyles) => {
            const state = {
                screens: [...this.state.screens, { name, params, transitionStyles }],
                active: this.state.screens.length,
            };
            this.setState(state, () => {
                this.replaceState({ name, params });
            });
        };
        this.pop = (params = { modifyHistory: true }) => {
            const state = {
                screens: [...this.state.screens.slice(0, this.state.screens.length - 1)],
                active: this.state.active - 1,
            };
            this.setState(state, () => {
                const screen = state.screens[state.screens.length - 1];
                this.replaceState(screen);
            });
        };
        this.popToRoot = () => {
            throw new Error('unimplemented');
        };
        this.resetTo = () => {
            throw new Error('unimplemented');
        };
        this.showModal = () => {
            throw new Error('unimplemented');
        };
        this.dismissModal = () => {
            throw new Error('unimplemented');
        };
        this.updateParams = (newParams, merge = true) => {
            const { name, params } = this.state.screens[this.state.screens.length - 1];
            this.replaceState({
                name,
                params: Object.assign({}, merge && params, newParams),
            });
        };
        this.name = props.name;
        this.state = {
            screens: [props.launcherScreen || props.homeScreen],
            active: 0,
        };
        context.application.registerNavigator(this);
    }
    getChildContext() {
        return {
            navigator: {
                name: this.name,
                parent: this.parent,
                push: this.push,
                pop: this.pop,
                popToRoot: this.popToRoot,
                resetTo: this.resetTo,
                showModal: this.showModal,
                dismissModal: this.dismissModal,
                updateParams: this.updateParams,
                finishLaunching: this.finishLaunching,
            },
        };
    }
    render() {
        const { style, className } = this.props;
        return (React.createElement(react_transition_group_1.TransitionGroup, { className: classNames('react-navigation-navigator', className), style: style }, this.state.screens.map(this.renderScreen)));
    }
    replaceState(screen) {
        const { name, params } = screen;
        let url = `?page=${name}`;
        if (Object.keys(params || {}).length) {
            url += `&params=${JSON.stringify(params)}`;
        }
        history.replaceState({}, '', url);
    }
}
Navigator.childContextTypes = {
    navigator: PropTypes.object.isRequired,
};
Navigator.contextTypes = {
    registry: PropTypes.object.isRequired,
    application: PropTypes.object.isRequired,
};
exports.Navigator = Navigator;
